#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAX_LINE_LENGTH 100

typedef struct Node {
    char text[MAX_LINE_LENGTH];
    struct Node *yes;
    struct Node *no;
} Node;

Node* create_node(const char* text) {
    Node *node = (Node*)malloc(sizeof(Node));
    if (!node) {
        fprintf(stderr, "Memory allocation error.\n");
        exit(1);
    }
    strncpy(node->text, text, MAX_LINE_LENGTH);
    node->yes = NULL;
    node->no = NULL;
    return node;
}

Node* parse_knowledge_base(FILE *file) {
    char line[MAX_LINE_LENGTH];
    
    if (!fgets(line, sizeof(line), file)) {
        return NULL;
    }

    line[strcspn(line, "\n")] = 0;

    if (line[0] == '*') {
        return create_node(line + 1);
    } else {
        Node *node = create_node(line);
        node->yes = parse_knowledge_base(file);
        node->no = parse_knowledge_base(file);
        return node;
    }
}

int count_products(Node *node) {
    if (!node) return 0;
    if (!node->yes && !node->no) return 1;
    return count_products(node->yes) + count_products(node->no);
}

void run_expert_system(Node *node) {
    while (node) {
        if (!node->yes && !node->no) {
            printf("The expert knows: %s.\n", node->text);
            return;
        }
        printf("%s\n", node->text);
        printf("Answer 'a' for the first option or 'n' for the second option: ");
        char answer;
        scanf(" %c", &answer);
        if (answer == 'a') {
            node = node->yes;
        } else if (answer == 'n') {
            node = node->no;
        } else {
            printf("Invalid answer. Exiting.\n");
            return;
        }
    }
}

void create_default_knowledge_base() {
    FILE *file = fopen("knowledge_base.txt", "w");
    if (!file) {
        perror("Failed to create the knowledge base file");
        exit(1);
    }
    fprintf(file, "Je to ovocie alebo zelenina?\n");
    fprintf(file, "*Jablko\n");
    fprintf(file, "*Mrkva\n");
    fclose(file);
    printf("Default knowledge base file 'knowledge_base.txt' has been created.\n");
}

int main() {
    FILE *file = fopen("knowledge_base.txt", "r");
    if (!file) {
        printf("Knowledge base file not found. Creating default file.\n");
        create_default_knowledge_base();
        file = fopen("knowledge_base.txt", "r");
        if (!file) {
            perror("Failed to open the newly created knowledge base file");
            return 1;
        }
    }

    Node *root = parse_knowledge_base(file);
    fclose(file);

    if (!root) {
        printf("The knowledge base could not be loaded.\n");
        return 1;
    }

    int product_count = count_products(root);
    printf("The system knows about %d types of fruits and vegetables.\n", product_count);
    
    run_expert_system(root);

    free(root);
    return 0;
}

